package net.f00f.javathrottle;
/*
	JavaThrottle, a program to limit the bandwidth between an input
	stream and an output stream.  Copyright (C) 2003 Ben Damm

	For information, contact bdamm@f00f.net

                        -----  LGPL License  -----
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                        -----       END      -----

*/


import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.util.Map;
import java.util.Properties;
import java.util.Observer;
import java.util.Observable;

import javax.swing.*;

/**
 * This class does the command line switches and the GUI.  The
 * throttling of the pipe is done in a Throttle implementation.
 *
 * @author Ben Damm
 * */
// Rates are bytes per millisecond, throughout the code.
public class Main extends JPanel implements ActionListener
{
	private static String KEY_desiredRate = "desiredRate";

	private JLabel bytesLabel;
	private JLabel rateLabel;
	private JLabel timeLabel;
	private JLabel bufLabel;
	private JLabel windowLabel;
	private JLabel desiredWindowLabel;
	private JLabel windowMaxReadLabel;

	private Throttle throttle;

	private static String VER_project = "JavaThrottle";
	private static String VER_version = Version.VERSION + " " + Version.BUILD_DATE;
	private static String VER_author = "Ben Damm, bdamm@f00f.net";
	private static String VER_url = "http://dammfine.com/projects/javathrottle/";

	private static boolean DEBUG_enable = false;

	/**
	 * Entry point.
	 * */
	public static void main( String args[] )
	{
		// Parse cmd line
		Properties p = parseCmdLine( args );

		// Error
		if( p == null )
		{
			usage();
			return;
		}

		// The window skeleton
		JFrame jf = new JFrame("JavaThrottle");

		// When the user closes the window, kill the application.
		jf.addWindowListener(new WindowAdapter()
				{
					public void windowClosing( WindowEvent we )
					{
						System.exit(0);
					}
				});

		// Create the main class
		Main mainPane = new Main(p);

		// Add it
		jf.getContentPane().add( mainPane );
		jf.pack();
		jf.show();

		// Start the magic
		mainPane.run();
	}


	/**
	 *
	 * */
	public Main( Map props )
	{
		BlockInputStream in = new BlockInputStream( System.in, OutThrottle.MAX_bufsize );
		throttle = new OutThrottle( in, System.out );

		throttle.setDesiredRate( (Rate)props.get(KEY_desiredRate) );

		throttle.addObserver( new Observer()
				{
					public void update( Observable o1, Object o2 )
					{
						updateLabels();
					}
				});

		int rows = 0;

		add( new JLabel("Bytes: " ));
		add( bytesLabel = new JLabel("-") );
		++rows;

		add( new JLabel("Rate: " ));
		add( rateLabel = new JLabel("-") );
		++rows;

		add( new JLabel("Desired rate: " ));
		JRateSelect jrs = new JRateSelect(throttle.getDesiredRate());
		jrs.addActionListener( this );
		add( jrs );
		++rows;

		JComponent comp;

		add( comp = new JLabel("Window:"));
		comp.setVisible(true);
		add( comp = windowLabel = new JLabel("-") );
		comp.setVisible(true);
		++rows;

		add( new JLabel("Desired Window:") );
		JWindowSelect jws = new JWindowSelect((int)throttle.getDesiredWindow());
		jws.addActionListener( this );
		add( jws );
		++rows;

		add( new JLabel("Buffer:"));
		add( bufLabel = new JLabel("-") );
		++rows;

		// The biggest read performed in the window
		add( new JLabel("Max read:" ) );
		add( windowMaxReadLabel = new JLabel("-") );
		++rows;

		add( new JLabel("Time: " ) );
		add( timeLabel = new JLabel("-") );
		++rows;
		
		JButton pauseBtn = new JButton("Pause");
		pauseBtn.addActionListener( new ActionListener()
				{
					public void actionPerformed( ActionEvent ae )
					{
						throttle.togglePaused();
					}
				});
		add( pauseBtn );
		JButton stopBtn = new JButton( "Stop" );
		stopBtn.addActionListener( new ActionListener()
				{
					public void actionPerformed( ActionEvent ae )
					{
						//throttle.shutdown();
						//try { Thread.sleep(2000); } catch ( InterruptedException ie ) {}
						System.exit(0);
					}
				});
		add( stopBtn );
		++rows;

		setLayout(new GridLayout(rows,2));
	}

	public void actionPerformed( ActionEvent ae )
	{
		if( ae instanceof NewRateEvent )
		{
			NewRateEvent nre = (NewRateEvent)ae;

			throttle.setDesiredRate(nre.getNewRate());
		}
		else if( ae instanceof NewWindowEvent )
		{
			NewWindowEvent nwe = (NewWindowEvent)ae;

			throttle.setDesiredWindow(nwe.getNewWindow());
		}
	}

	public void run()
	{
		if( throttle instanceof Thread )
		{
			((Thread)throttle).start();
		}
	}

	public static void usage()
	{
		log( "Usage: " );
		log( "   java -jar javathrottle.jar <options>" );
		log( "" );
		log( " Options:" );
		log( "  -b, -k, -m <number>" );
		log( "      Limit pipe in bits, kilobits, or megabits per second" );
		log( "  -B, -K, -M <number>" );
		log( "      Limit pipe in bytes, kilobytes, or megabytes per second" );
		log( "  -U" );
		log( "      Start with throttle wide open" );
		log( "  --version " );
		log( "      Show version number ");

	}

	public static void debug(String msg)
	{
		if( DEBUG_enable )
			System.err.println(msg);
	}

	public void stop()
	{
		//throttle.shutdown();
		System.exit(0);
	}

	// ------------ private methods --------------
	private static Properties parseCmdLine( String args[] )
	{
		Properties p = new Properties();

		Unit unit = null;
		double value = 1;

		for( int i = 0; i < args.length; ++i )
		{
			if( args[i].equals( "--version" ) )
			{
				log( VER_project + " " + VER_version );
				log( VER_author );
				log( VER_url );
				System.exit(0);
			}
			// Any argument must be in the form "-X"
			if( args[i].length() < 2 ) 
				return null;

			// If there are any flags, they must start with "-"
			if( args[i].charAt(0) != '-' )
				return null;

			char c = args[i].charAt(1);
			boolean readRate = false;
			if( readRate = c == 'b' )
				unit = Unit.bps;
			else if( readRate = c == 'k' )
				unit = Unit.kbps;
			else if ( readRate = c == 'm' )
				unit = Unit.Mbps;
			else if( readRate = c == 'B' )
				unit = Unit.Bps;
			else if( readRate = c == 'K' )
				unit = Unit.kBps;
			else if( readRate = c == 'M' )
				unit = Unit.MBps;
			else if( c == 'U' )
			{
				unit = Unit.unlimited;
				p.put( KEY_desiredRate, unit.makeRate( 1 ) );
			}
			else return null;

			if( readRate )
			{
				p.put( KEY_desiredRate, unit.makeRate( args[++i] ) );
			}
		}

		return p;
	}

	private static void log(String msg)
	{
		System.err.println(msg);
	}

	private void updateLabels()
	{
		bytesLabel.setText( "" + throttle.getTotalBytes() );
		rateLabel.setText( "" + throttle.getActualRate().toString() );
		bufLabel.setText( "" + throttle.getActualBufSize() );
		windowLabel.setText( "" + throttle.getActualWindow() + " 1/1000 sec");
		windowMaxReadLabel.setText( "" + throttle.getMaxWindowRead() );

		// Time label
		long now = System.currentTimeMillis();
		long totalTime = throttle.getTotalTime();
		if( totalTime > 0 )
			timeLabel.setText( elapsedTimeToStr( totalTime ) );
	}


	private String elapsedTimeToStr( long elapsed )
	{
		long seconds = elapsed / 1000;
		long minutes = seconds / 60;
		long hours = minutes / 60;
		long days = hours / 24;

		seconds = seconds % 60;
		minutes = minutes % 60;
		hours = hours % 24;

		StringBuffer sb = new StringBuffer(20);
		boolean div = false;
		div = appendTimeUnit( sb, days, " day", " days", div );
		div = appendTimeUnit( sb, hours, " hour", " hours", div );
		div = appendTimeUnit( sb, minutes, " minute", " minutes", div );
		div = appendTimeUnit( sb, seconds, " second", " seconds", div );

		return sb.toString();
	}

	private boolean appendTimeUnit( StringBuffer sb, long data, String unary, String plural, boolean prefixDelim )
	{
		if( data > 0 )
		{
			if( prefixDelim )
				sb.append( ", " );

			sb.append( "" + data );
			if( data > 1 )
				sb.append( plural );
			else
				sb.append( unary );
			return true;
		}
		else
			return prefixDelim;
	}
}

